using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using Utils.Logs;
using Utils.Misc;

namespace Components.Calendar.MonthCal.Class
{
    /// <summary>
    /// Gets the vLanguage of the Calendar
    /// </summary>
    public class CalendarLanguage
    {
        #region Variables
        /// <summary>
        /// Indicates the path of the XMl vLanguage file
        /// </summary>
        private string vXmlPath = null;
        /// <summary>
        /// Name of days
        /// </summary>
        private string[] vNameOfDays = { "Monday", "Tuesday", "Wednesday", "Thrusday", "Friday", "Saterday", "Sunday" };
        /// <summary>
        /// Name of months
        /// </summary>
        private string[] vNameOfMonths = { "January", "February", "March", "April", "May", "June", "July", "August",
            "September", "October", "November", "December" };
        /// <summary>
        /// Array of the different languages
        /// </summary>
        private string[] vLanguages = { "English" };
        /// <summary>
        /// Indicates the index of the selected vLanguage
        /// </summary>
        public int vSelectedLanguage = 0;
        /// <summary>
        /// vXmlDocument vLanguage file
        /// </summary>
        XmlDocument vXmlDocument = null;
        /// <summary>
        /// The vLanguage nodes which contains the names of the days and the months
        /// </summary>
        XmlNodeList vLanguagesNodes = null;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor. Try to load the vXmlDocument vLanguage file in memory. If loading failed the default vLanguage is english
        /// </summary>
        /// <param name="_XMLPath">Path of the vXmlDocument vLanguage file</param>
        public CalendarLanguage(string _XMLPath)
        {
            this.vXmlPath = _XMLPath;
            this.TryLoad();
        }
        /// <summary>
        /// Constructor. Do not load any file in memory but load the english vLanguage
        /// </summary>
        public CalendarLanguage() { }
        #endregion

        #region Methods
        /// <summary>
        /// Fill the instance with a new instance of CalendarLanguage
        /// </summary>
        /// <param name="_CalendarLanguage"></param>
        public void FillFrom(CalendarLanguage _CalendarLanguage)
        {
            this.vXmlPath = _CalendarLanguage.XmlPath;
            this.TryLoad();
        }
        /// <summary>
        /// Try to load the vLanguage list of the property XmlPath
        /// </summary>
        /// <returns>TRUE if the load succeeded or FALSD loading failed</returns>
        public bool TryLoad()
        {
            if (this.vXmlPath == null || !File.Exists(this.vXmlPath)) return false;

            try
            {
                vXmlDocument = new XmlDocument();
                vXmlDocument.Load(this.vXmlPath);
                vLanguagesNodes = vXmlDocument.ChildNodes[1].ChildNodes;

                vLanguages = new string[vLanguagesNodes.Count];

                for (int i = 0; i < vLanguages.Length; i++)
                    vLanguages[i] = TextManager.FirstUpper(vLanguagesNodes[i].Attributes[0].Value);
                return true;
            }
            catch (Exception ex)
            {
                vXmlDocument = null;
                Log.Instance.Write(ex.Message, this, LogType.Error);
                return false;
            }
        }
        /// <summary>
        /// Load the vLanguage list of the property XmlPath or throw an exception if load failed
        /// </summary>
        ///<exception cref="Exception">If an error occured</exception>
        ///<exception cref="XmlException">If the Load of the Xml file failed</exception>
        ///<exception cref="FileNotFoundException">If the Xml vLanguage file was not found</exception>
        public void Load()
        {
            if (this.vXmlPath == null || !File.Exists(this.vXmlPath))
                throw new FileNotFoundException("The Xml language file was not found!");

            try
            {
                vXmlDocument = new XmlDocument();
                vXmlDocument.Load(this.vXmlPath);
                vLanguagesNodes = vXmlDocument.ChildNodes[1].ChildNodes;
                vLanguages = new string[vLanguagesNodes.Count];

                for (int i = 0; i < vLanguages.Length; i++)
                    vLanguages[i] = TextManager.FirstUpper(vLanguagesNodes[i].Attributes[0].Value);
            }
            catch (XmlException ex)
            {
                vXmlDocument = null;
                Log.Instance.Write(ex.Message, this, LogType.Error);
                throw ex;
            }
            catch (Exception ex)
            {
                vXmlDocument = null;
                Log.Instance.Write(ex.Message, this, LogType.Error);
                throw ex;
            }
        }
        /// <summary>
        /// Load the default vLanguage (English) in the object
        /// </summary>
        public void LoadDefault()
        {
            vLanguages = new string[] { "English" };
            vNameOfDays = new string[] { "Monday", "Tuesday", "Wednesday", "Thrusday", "Friday", "Saterday", "Sunday" };
            vNameOfMonths = new string[]{ "January", "February", "March", "April", "May", "June", "July", "August",
            "September", "October", "November", "December" };
        }
        /// <summary>
        /// Try to select a vLanguage.
        /// </summary>
        /// <param name="_Index">The index of the vLanguage array (Cf. property Languages)</param>
        /// <returns>TRUE if the selection succeeded or FALSE if it failed 
        /// (Failed if the index if less than 0 or bigger than the number of languages)</returns>
        public bool TrySelectLanguage(int _Index)
        {
            if (_Index < 0 || _Index > this.vLanguages.Length) return false;
            this.vSelectedLanguage = _Index;
            return true;
        }
        /// <summary>
        /// Try to select a vLanguage.
        /// </summary>
        /// <param name="_CalendarLanguage">The name of the vLanguage</param>
        /// <returns>TRUE if the selection succeeded or FALSE if it failed 
        /// (Failed if the index if less than 0 or bigger than the number of languages)</returns>
        public bool TrySelectLanguage(string _CalendarLanguage)
        {
            int vIndex = -1;
            _CalendarLanguage = Utils.Misc.TextManager.FirstUpper(_CalendarLanguage);
            for (int i = 0; i < this.vLanguages.Length; i++)
            {
                if (vLanguages[i] == _CalendarLanguage)
                {
                    vIndex = i;
                    break;
                }
            }
            if (vIndex < 0) return false;
            else
            {
                this.vSelectedLanguage = vIndex;
                return true;
            }


        }
        /// <summary>
        /// Select a vLanguage or throw an exception if selection is impossible
        /// </summary>
        /// <param name="_Index">The index of the vLanguage array (Cf. property Languages)</param>
        /// <returns>TRUE if the selection succeeded or FALSE if it failed 
        /// (Failed if the index if less than 0 or bigger than the number of languages)</returns>
        /// <exception cref="IndexOutOfRangeException">If the index is less than zero or bigger 
        /// than the vLanguage list</exception>
        public void SelectLanguage(int _Index)
        {
            if (_Index < 0 || _Index >= this.vLanguages.Length)
                throw new IndexOutOfRangeException("The language index is out of range or less than zero");

            this.vSelectedLanguage = _Index;            
            this.OnLanguageChanged();
        }
        /// <summary>
        /// Select a vLanguage or throw an exception if selection is impossible
        /// </summary>
        /// <param name="_CalendarLanguage">The name of the vLanguage</param>
        /// <returns>TRUE if the selection succeeded or FALSE if it failed 
        /// (Failed if the index if less than 0 or bigger than the number of languages)</returns>
        /// <exception cref="ArgumentException">If the vLanguage does not exist in the list</exception>
        public void SelectLanguage(string _CalendarLanguage)
        {
            int vIndex = -1;
            if (vLanguages == null)
                this.Load();
            for (int i = 0; i < this.vLanguages.Length; i++)
            {
                if (vLanguages[i].ToLower() == _CalendarLanguage.ToLower())
                {
                    vIndex = i;
                    break;
                }
            }
            if (vIndex < 0)
                throw new ArgumentException("The language \"" + _CalendarLanguage + "\" is not supported!");

            this.vSelectedLanguage = vIndex;            
            this.OnLanguageChanged();

        }
        /// <summary>
        /// Returns a string which contains the selected language
        /// </summary>
        /// <returns>The selected language</returns>
        public override string ToString() { return TextManager.FirstUpper(this.vLanguages[vSelectedLanguage]); }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the path of the vXmlDocument vLanguage file
        /// </summary>
        public string XmlPath { get { return this.vXmlPath; } set { this.vXmlPath = value; } }
        /// <summary>
        /// Gets the list which conatins the name of the months
        /// </summary>
        public string[] Months
        {
            get
            {
                if (vXmlDocument == null) return this.vNameOfMonths;
                if (this.vLanguagesNodes == null) return this.vNameOfMonths;
                if (this.vSelectedLanguage < 0 || this.vSelectedLanguage >= this.vLanguages.Length)
                    return this.vNameOfMonths;

                //Buf contains the months name separated with coma ","
                string Buf = vLanguagesNodes[this.vSelectedLanguage].ChildNodes[1].InnerText;
                Buf.Trim();
                return Buf.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            }
        }
        /// <summary>
        /// Gets the list which contains the name of the days
        /// </summary>
        public string[] Days
        {
            get
            {
                if (vXmlDocument == null) return this.vNameOfDays;
                if (this.vLanguagesNodes == null) return this.vNameOfDays;
                if (this.vSelectedLanguage < 0 || this.vSelectedLanguage > this.vLanguages.Length)
                    return this.vNameOfDays;

                //Buf contains the days name separated with coma ","
                string Buf = vLanguagesNodes[this.vSelectedLanguage].ChildNodes[0].InnerText;
                Buf.Trim();
                return Buf.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            }
        }
        /// <summary>
        /// Gets the list of the supported languages.
        /// </summary>
        public string[] Languages { get { return this.vLanguages; } }
        /// <summary>
        /// Gets the selected language
        /// </summary>
        public string SelectedLanguage { get { return this.vLanguages[this.vSelectedLanguage]; } }
        /// <summary>
        /// Gets the selected language index
        /// </summary>
        public int SelectedLanguageIndex { get { return this.vSelectedLanguage; } }
        /// <summary>
        /// Indicates if the Language is already set
        /// </summary>
        public bool AlreadySet
        {
            get
            {
                try
                {
                    if (this.vXmlDocument != null)
                        return bool.Parse(this.vXmlDocument.ChildNodes[1].ChildNodes[1].Value);
                    else
                        return false;
                }
                catch (Exception ex)
                {
                    Log.Instance.Write(ex.Message, this, LogType.Error);
                    return false;
                }
            }
        }
        #endregion

        #region Event
        /// <summary>
        /// Event thrown when the language changed
        /// </summary>
        public event EventHandler LanguageChanged;
        /// <summary>
        /// Notify a language changed
        /// </summary>
        protected virtual void OnLanguageChanged()
        {
            if (LanguageChanged != null)
                LanguageChanged(this, new EventArgs());
        }
        #endregion
    }
}
