using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Components.Calendar.MonthCal;
using Components.Calendar.MonthCal.Class.Event;
using Components.Calendar.MonthCal.Class;
using System.Drawing.Design;
using Components.TypeEditors;
using Utils.Misc;

namespace Components.Calendar.MonthCal.GUI
{
    /// <summary>
    /// Component which represents a month calendar
    /// </summary>
    public partial class MonthCal : UserControl
    {
        #region Variables
        /// <summary>
        /// Informations about the days to display
        /// </summary>
        private Days vDays;
        /// <summary>
        /// Height of the upper header
        /// </summary>
        private const int vHeaderHeight = 40;
        /// <summary>
        /// The upper header which contains the actual month and the navigation menu
        /// </summary>
        private MonthRectangleInfo[] vHeader = new MonthRectangleInfo[3];
        /// <summary>
        /// Height of the middle header (Name of days)
        /// </summary>
        private const int vHeaderDayHeight = 30;
        /// <summary>
        /// The middle header which contains the names of the days of the week
        /// </summary>
        private MonthRectangleInfo[] vHeaderDays = new MonthRectangleInfo[7];
        /// <summary>
        /// Language of the calendar
        /// </summary>
        private CalendarLanguage vLanguage = new CalendarLanguage();
        /// <summary>
        /// The default cursor of the component
        /// </summary>
        private static Cursor vDefaultCursor;
        /// <summary>
        /// Index of the selected language
        /// </summary>
        private int vLanguageIndex = 0;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public MonthCal()
        {
            InitializeComponent();

            this.vLanguage.LoadDefault();
            this.vDays = new Days(this.pb_Agenda.Size, vHeaderHeight + vHeaderDayHeight);
            this.vDays.Resize();
            vDefaultCursor = this.Cursor;

            this.MouseWheel += new MouseEventHandler(Event_MouseWheel);
            this.vLanguage.LanguageChanged += new EventHandler(Event_LanguageChanged);

            #region Instanciate headers
            for (int i = 0; i < this.vHeader.Length; i++)
                if (vHeader[i] == null) vHeader[i] = new MonthRectangleInfo();

            Rectangle r = new Rectangle(new Point(0, vHeaderHeight),
                new Size(((pb_Agenda.Width) / 7) + (pb_Agenda.Width % 7) - 1, vHeaderDayHeight));

            for (int i = 0; i < this.vHeaderDays.Length; i++)
            {
                if (vHeaderDays[i] == null) vHeaderDays[i] = new MonthRectangleInfo();
                vHeaderDays[i].Rectangle = r;
                r = new Rectangle(new Point(r.X + r.Width, vHeaderHeight),
                    new Size((pb_Agenda.Width) / 7, vHeaderDayHeight));
                vHeaderDays[i].DayNumber = vLanguage.Days[i];
            }
            #endregion
        }
        #endregion

        #region Methods
        /// <summary>
        /// Sets the headers information (Color, Text, Size...) before displaying
        /// </summary>
        private void SetHeaders()
        {
            vHeader[0].Rectangle = new Rectangle(new Point(0, 0),
                new Size(51, vHeaderHeight));

            //Next et Previous son 10 ton plus bas que le reste
            Color MiddleColor = vHeader[2].BackgroundColor;
            int _R = MiddleColor.R;
            int _G = MiddleColor.R;
            int _B = MiddleColor.B;

            Color TColor;
            if (_R * _G * _B > 385)
                TColor = Color.Black;
            else
                TColor = Color.White;

            vHeader[0].BackgroundColor = vHeader[2].BackgroundColor;
            vHeader[0].TextColor = TColor;
            vHeader[0].TextFont = new Font("Times new Roman", 15);
            vHeader[0].BorderPen = new Pen(vHeader[2].BackgroundColor, 1);
            vHeader[0].DayNumber = "<<";

            vHeader[1].Rectangle = new Rectangle(new Point(51, 0),
                new Size(pb_Agenda.Width - 2 * 51, vHeaderHeight));

            vHeader[1].BorderPen = new Pen(vHeader[1].BackgroundColor, 1);

            vHeader[2].Rectangle = new Rectangle(new Point(pb_Agenda.Width - 51, 0),
                new Size(51, vHeaderHeight));
            vHeader[2].BackgroundColor = vHeader[1].BackgroundColor;
            vHeader[2].TextColor = TColor;
            vHeader[2].TextFont = new Font("Times new Roman", 15);
            vHeader[2].BorderPen = new Pen(vHeader[2].BackgroundColor, 1);
            vHeader[2].DayNumber = ">>";
        }
        /// <summary>
        /// Redraw the MonthCal
        /// </summary>
        /// <param name="g">Surface de dessin</param>
        protected void Redraw(Graphics g)
        {
            SetHeaders();

            #region Header
            //Gets the vLanguage information of the calendar
            if (!vLanguage.TryLoad()) vLanguage.LoadDefault();
            string[] month = vLanguage.Months;
            string[] day = vLanguage.Days;
            vHeader[1].DayNumber = month[vDays.DateToDisplay.Month - 1] + " " + vDays.DateToDisplay.Year;
            SizeF CoordString;

            for (int i = 0; i < 3; i++)
            {
                g.FillRectangle(new SolidBrush(vHeader[i].BackgroundColor), vHeader[i].Rectangle);
                g.DrawRectangle(vHeader[i].BorderPen, vHeader[i].Rectangle);

                CoordString = g.MeasureString(vHeader[i].DayNumber, vHeader[i].TextFont);

                CoordString.Width = ((vHeader[i].Rectangle.Width) / 2) - (CoordString.Width / 2);
                CoordString.Height = ((vHeader[i].Rectangle.Height) / 2) - (CoordString.Height / 2);

                if (i == 2)
                    CoordString.Width += pb_Agenda.Width - 51;

                g.DrawString(vHeader[i].DayNumber, vHeader[i].TextFont, new SolidBrush(vHeader[i].TextColor),
                    new Point((int)CoordString.Width, (int)CoordString.Height));
            }
            #endregion

            #region Header Days
            for (int i = 0; i < 7; i++)
            {
                vHeaderDays[i].DayNumber = day[i];
                g.FillRectangle(new SolidBrush(vHeaderDays[i].BackgroundColor), vHeaderDays[i].Rectangle);
                g.DrawRectangle(vHeaderDays[i].BorderPen, vHeaderDays[i].Rectangle);

                CoordString = g.MeasureString(vHeaderDays[i].DayNumber, vHeaderDays[i].TextFont);
                CoordString.Width = vHeaderDays[i].Rectangle.X
                    + (vHeaderDays[i].Rectangle.Width / 2) - (CoordString.Width / 2);

                CoordString.Height = vHeaderDays[i].Rectangle.Y
                    + (vHeaderDays[i].Rectangle.Height / 2) - (CoordString.Height / 2);

                g.DrawString(vHeaderDays[i].DayNumber, vHeaderDays[i].TextFont, new SolidBrush(vHeaderDays[i].TextColor),
                    new Point((int)CoordString.Width, (int)CoordString.Height));
            }
            #endregion

            #region Days
            vDays.Resize();

            //Dessinage des jours dans la calendrier            
            foreach (MonthRectangleInfo r in vDays)
            {
                //Affichage d'aujourd'hui
                if (vDays.TodayIndex.ToString() == r.DayNumber && vDays.IsActualMonth)
                    g.FillRectangle(new SolidBrush(vDays.TodayBackColor), r.Rectangle);
                else
                    g.FillRectangle(new SolidBrush(r.BackgroundColor), r.Rectangle);
                g.DrawRectangle(r.BorderPen, r.Rectangle);

                //On dessine les jours en haut  droite de la case
                CoordString = g.MeasureString(r.DayNumber, r.TextFont);
                CoordString.Width = r.Rectangle.X + (r.Rectangle.Width - CoordString.Width);
                //+ (r.Rectangle.Width / 2) - (CoordString.Width / 2);

                CoordString.Height = r.Rectangle.Y;
                //+ (r.Rectangle.Height / 2) - (CoordString.Height / 2);

                Font nf = new Font(r.TextFont, FontStyle.Bold);
                g.DrawString(r.DayNumber, r.TextFont, new SolidBrush(r.TextColor),
                    new Point((int)CoordString.Width, (int)CoordString.Height));

                //Affichage des nombre de rdv
                g.DrawString(r.Text, nf, new SolidBrush(r.TextColor),
                    new Point(r.Rectangle.X, (r.Rectangle.Y + r.Rectangle.Height)
                    - (int)g.MeasureString(r.Text, r.TextFont).Height));

                r.Text = "";
            }
            #endregion
        }
        /// <summary>
        /// Redraw the frame on a resizing
        /// </summary>        
        protected void Resized()
        {
            #region Resize de la fentre
            vHeader[1].Rectangle = new Rectangle(new Point(0, 0), new Size(pb_Agenda.Width - 1, vHeaderHeight));

            #region HeaderDays
            Rectangle r = new Rectangle(new Point(0, vHeaderHeight),
                new Size(((pb_Agenda.Width) / 7) + (pb_Agenda.Width % 7) - 1, vHeaderDayHeight));

            for (int i = 0; i < 7; i++)
            {
                vHeaderDays[i].Rectangle = r;
                r = new Rectangle(new Point(r.X + r.Width, vHeaderHeight),
                    new Size((pb_Agenda.Width) / 7, vHeaderDayHeight));
            }
            #endregion

            #region Days
            vDays.ComponentSize = pb_Agenda.Size;
            #endregion

            #endregion

            Graphics g = pb_Agenda.CreateGraphics();
            Redraw(g);
        }
        /// <summary>
        /// Check if the mouse click (MouseEventArgs) is on one the the navigation button
        /// </summary>
        /// <param name="e">Information about the mouse click </param>
        /// <param name="_ClickOnMenu">Indicates where the click was done (Next, Previous or Nowhere)</param>
        /// <returns>TRUE if the click was on a navigation menu otherwise FALSE</returns>
        protected bool IsCursorOnMonth(MouseEventArgs e, out ClickOnMenu _ClickOnMenu)
        {
            int zWidth = vHeader[0].Rectangle.Width;
            int zHeight = vHeader[0].Rectangle.Height;
            int zX = vHeader[0].Rectangle.X;
            int zY = vHeader[0].Rectangle.Y;

            if ((e.X > zX && e.X < zWidth + zX) && (e.Y > zY && e.Y < zHeight + zY))
            {
                _ClickOnMenu = ClickOnMenu.PreviousMonth;//0;
                return true;
            }

            zWidth = vHeader[2].Rectangle.Width;
            zHeight = vHeader[2].Rectangle.Height;
            zX = vHeader[2].Rectangle.X;
            zY = vHeader[2].Rectangle.Y;

            if ((e.X > zX && e.X < zWidth + zX) && (e.Y > zY && e.Y < zHeight + zY))
            {
                Cursor.Current = Cursors.Hand;
                _ClickOnMenu = ClickOnMenu.NextMonth;//2;
                return true;
            }

            zWidth = vHeader[1].Rectangle.Width;
            zHeight = vHeader[1].Rectangle.Height;
            zX = vHeader[1].Rectangle.X;
            zY = vHeader[1].Rectangle.Y;
            if ((e.X > zX && e.X < zWidth + zX) && (e.Y > zY && e.Y < zHeight + zY))
            {
                Cursor.Current = Cursors.Default;
                _ClickOnMenu = ClickOnMenu.MenuDate;
                return true;
            }
            else
            {
                _ClickOnMenu = ClickOnMenu.Nothing;
                return false;
            }
        }
        /// <summary>
        /// Check if the mouse is on a DayRectangle
        /// </summary>
        /// <param name="e">Mouse information</param>  
        /// <param name="_MonthRectangleInfo">Contains the information about the days wheron the mouse was 
        /// or NULL if the mouse was not on a day</param>
        /// <returns>TRUE of the mouse was on a day otherwise FALSE</returns>
        protected bool IsCursorOnDay(MouseEventArgs e, out MonthRectangleInfo _MonthRectangleInfo)
        {
            foreach (MonthRectangleInfo r in vDays)
            {
                if (r.Rectangle.Contains(e.Location))
                {
                    _MonthRectangleInfo = r;
                    return true;
                }
            }
            _MonthRectangleInfo = null;
            return false;
        }
        /// <summary>
        /// Displays the current month
        /// </summary>
        public void DisplayToday()
        {
            DateTime dtBuf = this.vDays.DateToDisplay;
            this.vDays.DateToDisplay = DateTime.Today;
            OnMonthChanged(new MonthChangedEventArgs(dtBuf, this.vDays.DateToDisplay));
        }
        /// <summary>
        /// Give the focus to the MonthCal
        /// </summary>
        public void Focus() { pb_Agenda.Focus(); }
        #endregion

        #region Properties
        /// <summary>
        /// Donne la liste des remarques des rendez-vous
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        public IMeetingRowList MeetingSource
        {
            get
            {
                RootMeetingSet mr = new RootMeetingSet(vDays.MeetingList);
                return mr as IMeetingRowList;
            }
            set
            {
                vDays.MeetingList = value.MeetingList;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }

        #region Language
        /// <summary>
        /// Gets or sets the selected language for the calendar
        /// </summary>
        [Browsable(true)]
        [Category("Options - Language")]
        [Description("Language of the calendar")]
        [TypeConverter(typeof(CalendarLanguageConverter))]
        [Editor(typeof(UICalendarEditor), typeof(UITypeEditor))]        
        public string SelectedLanguage
        {
            get { return this.vLanguage.SelectedLanguage; }
            set { this.vLanguage.SelectLanguage(value); }
        }
        /// <summary>
        /// Gets the language manager
        /// </summary>
        [Browsable(false)]
        public CalendarLanguage Language { get { return this.vLanguage; } }
        /// <summary>
        /// Gets or sets the language of the Calendar
        /// </summary>
        [Category("Options - Language")]
        [Description("Language file of the calendar")]
        [Editor(typeof(UIPathEditor), typeof(UITypeEditor))]
        [Browsable(true)]            
        public string LanguageFile
        {
            get { return vLanguage.XmlPath; }
            set
            {
                vLanguage.XmlPath = value;
                vLanguage.TryLoad();
                //vLanguage.SelectLanguage(this.vLanguageIndex);
            }
        }
        #endregion

        #region Header
        /// <summary>
        /// Couleur du texte du header
        /// </summary>
        [Description("The color of the upper header text")]
        [Category("Options - Header")]
        public Color HeaderTextColor
        {
            get { return vHeader[1].TextColor; }
            set
            {
                vHeader[1].TextColor = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Police du header
        /// </summary>
        [Category("Options - Header")]
        [Description("Font of the header text")]
        public Font HeaderTextFont
        {
            get { return vHeader[1].TextFont; }
            set
            {
                if (value.Size > 32)
                    value = new Font(value.Name, 32);
                vHeader[1].TextFont = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Couleur de fond du header
        /// </summary>
        [Category("Options - Header")]
        [Description("Background color of the header")]
        public Color HeaderBackgroundColor
        {
            get { return vHeader[1].BackgroundColor; }
            set
            {
                for (int i = 0; i < vHeader.Length; i++)
                    vHeader[i].BackgroundColor = value;

                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        #endregion

        #region Header Day
        /// <summary>
        /// Couleur du texte du header de jour
        /// </summary>
        [Description("Color of text of the header of day")]
        [Category("Options - HeaderDay")]
        public Color HeaderDayTextColor
        {
            get { return vHeaderDays[1].TextColor; }
            set
            {
                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].TextColor = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Police du header de jour
        /// </summary>
        [Category("Options - HeaderDay")]
        [Description("Font of the header day text")]
        public Font HeaderDayTextFont
        {
            get { return vHeaderDays[1].TextFont; }
            set
            {
                if (value.Size > 20)
                    value = new Font(value.Name, 20);

                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].TextFont = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Couleur de fond du header de jour
        /// </summary>
        [Category("Options - HeaderDay")]
        [Description("Background color of the header day")]
        public Color HeaderDayBackgroundColor
        {
            get { return vHeaderDays[1].BackgroundColor; }
            set
            {
                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].BackgroundColor = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Couleur de contour du header de jour
        /// </summary>
        [Category("Options - HeaderDay")]
        [Description("Color of the Header of days Border")]
        [Browsable(true)]
        public Color HeaderBorderDay
        {
            get { return vHeaderDays[1].BorderPen.Color; }
            set
            {
                Pen Value = new Pen(value);
                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].BorderPen = Value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        #endregion

        #region Days
        /// <summary>
        /// Retourne ou initialise la couleur d'criture des jours
        /// </summary>
        [Category("Options-Days")]
        [Description("Color of the text of the days")]
        public Color DaysTextColor
        {
            get { return vDays.ColorText; }
            set
            {
                vDays.ColorText = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Retourne ou initialise la police d'affichage des jours
        /// </summary>
        [Category("Options-Days")]
        [Description("Font of the days")]
        public Font DaysTextFont
        {
            get { return vDays.Font; }
            set
            {
                vDays.Font = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond des jours
        /// </summary>
        [Category("Options-Days")]
        [Description("Background color of the days")]
        public Color DaysBackColor
        {
            get { return vDays.ColorBack; }
            set
            {
                vDays.ColorBack = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond des jours hors du mois
        /// </summary>
        [Category("Options-Days")]
        [Description("Background color of the days which are out of the month")]
        public Color DaysNonInMonthColor
        {
            get { return vDays.ColorNonMonthBack; }
            set
            {
                vDays.ColorNonMonthBack = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur des colonnes du tableau des jours
        /// </summary>
        [Category("Options-Days")]
        [Description("Color of the columns border (which separates the days)")]
        public Color DaysBorderColor
        {
            get { return vDays.ColorSurround; }
            set
            {
                vDays.ColorSurround = value;
                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].BorderPen = new Pen(value);
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        /// <summary>
        /// Gets or sets the word which represents the couted stuff in the calendar
        /// </summary>
        [Category("Options")]
        [Description("The word wich represents the counted stuffs")]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        public string CountedWord { get { return vDays.CountedWord.Trim(); } set { vDays.CountedWord = value; } }
        /// <summary>
        /// Retourne ou initialise la date a afficher
        /// </summary>
        [Category("Options-Days")]
        [Description("Date to display")]
        [Browsable(false)]
        public DateTime DateToDisplay
        {
            get { return vDays.DateToDisplay; }
            set
            {
                vDays.DateToDisplay = value;
                Redraw(pb_Agenda.CreateGraphics());
            }
        }
        /// <summary>
        /// Retourne ou initialise le couleur de fond d'aujourd'hui
        /// </summary>
        [Category("Options-Today")]
        [Description("Background color of Today")]
        public Color TodayBackColor
        {
            get { return vDays.TodayBackColor; }
            set
            {
                vDays.TodayBackColor = value;
                Redraw(pb_Agenda.CreateGraphics()); ;
            }
        }
        #endregion
        #endregion

        #region Event
        /// <summary>
        /// Paint event of the component
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_PictureBox_Paint(object sender, PaintEventArgs e) { this.Redraw(e.Graphics); }
        /// <summary>
        /// Check a click on the calendar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_PictureBox_MouseDown(object sender, MouseEventArgs e)
        {
            #region Click on Next-Previous Month
            ClickOnMenu vClickOnMenu;
            if (this.IsCursorOnMonth(e, out vClickOnMenu))
            {
                switch (vClickOnMenu)
                {
                    case ClickOnMenu.NextMonth:
                        this.vDays.DateToDisplay = vDays.DateToDisplay.AddMonths(1);
                        this.Redraw(pb_Agenda.CreateGraphics());
                        this.OnMonthChanged
                            (new MonthChangedEventArgs(vDays.DateToDisplay.AddMonths(-1), vDays.DateToDisplay));
                        //MessageBox.Show("Next");
                        break;
                    case ClickOnMenu.PreviousMonth:
                        this.vDays.DateToDisplay = vDays.DateToDisplay.AddMonths(-1);
                        this.Redraw(pb_Agenda.CreateGraphics());
                        this.OnMonthChanged
                            (new MonthChangedEventArgs(vDays.DateToDisplay.AddMonths(1), vDays.DateToDisplay));
                        //MessageBox.Show("Previous");
                        break;
                    case ClickOnMenu.MenuDate:
                        DateTime Before = this.DateToDisplay;
                        fr_DateSelector fr = new fr_DateSelector();
                        fr.DateToDisplay = this.vDays.DateToDisplay;
                        fr.Language = this.vLanguage;
                        DialogResult dr = fr.ShowDialog();
                        if (dr == DialogResult.Cancel) return;
                        vDays.DateToDisplay = fr.DateToDisplay;
                        this.Redraw(pb_Agenda.CreateGraphics());
                        this.OnMonthChanged(new MonthChangedEventArgs(Before, this.DateToDisplay));
                        break;
                }
            }
            #endregion

            #region Click on Day
            MonthRectangleInfo r;
            if (IsCursorOnDay(e, out r))
            {
                if (e.Button == MouseButtons.Left && r != null)
                {
                    if (r.Position == CasePositionInMonth.InMonth)
                    {
                        OnDayClicked(new DayClickedEventArgs(new DateTime(this.vDays.DateToDisplay.Year,
                            this.vDays.DateToDisplay.Month, Int32.Parse(r.DayNumber))));
                    }
                    else if (r.Position == CasePositionInMonth.After)
                    {
                        this.vDays.DateToDisplay = this.vDays.DateToDisplay.AddMonths(1);
                        this.Redraw(pb_Agenda.CreateGraphics());
                        this.OnMonthChanged
                            (new MonthChangedEventArgs(vDays.DateToDisplay.AddMonths(-1), vDays.DateToDisplay));
                        OnMonthChanged(new MonthChangedEventArgs(vDays.DateToDisplay, vDays.DateToDisplay.AddMonths(1)));
                    }
                    else
                    {
                        this.vDays.DateToDisplay = this.vDays.DateToDisplay.AddMonths(-1);
                        this.Redraw(pb_Agenda.CreateGraphics());
                        OnMonthChanged(new MonthChangedEventArgs(vDays.DateToDisplay, vDays.DateToDisplay.AddMonths(-1)));
                    }
                }
            }
            #endregion

            #region Click out of month
            #endregion
        }
        /// <summary>
        /// The MonthCal was resized
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_PictureBox_SizeChanged(object sender, EventArgs e)
        {
            this.Resized();
        }
        /// <summary>
        /// The mouse moved on the PictureBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_PictureBox_MouseMove(object sender, MouseEventArgs e)
        {
            ClickOnMenu _ClickOnMenu;
            this.IsCursorOnMonth(e, out _ClickOnMenu);

            switch (_ClickOnMenu)
            {
                case ClickOnMenu.NextMonth:
                case ClickOnMenu.PreviousMonth:
                    this.Cursor = Cursors.Hand;
                    break;
                case ClickOnMenu.Nothing:
                    this.Cursor = vDefaultCursor;
                    break;
            }
        }
        /// <summary>
        /// MouseWheel event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseWheel(object sender, MouseEventArgs e)
        {
            if (e.Delta > 0)
            {
                DateTime Before = this.DateToDisplay;
                this.DateToDisplay = this.DateToDisplay.AddMonths(1);
                this.OnMonthChanged(new MonthChangedEventArgs(Before, this.DateToDisplay));
            }
            else if (e.Delta < 0)
            {
                DateTime Before = this.DateToDisplay;
                this.DateToDisplay = this.DateToDisplay.AddMonths(-1);
                this.OnMonthChanged(new MonthChangedEventArgs(Before, this.DateToDisplay));
            }
        }
        /// <summary>
        /// The calendar language changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Event_LanguageChanged(object sender, EventArgs e)
        {
            this.Redraw(this.pb_Agenda.CreateGraphics());
        }
        #endregion

        #region New Events
        /// <summary>
        /// Event thrown when the displayed month changed
        /// </summary>
        [Category("MonthCal - Event")]
        [Description("Event thrown when the displayed month changed")]
        public event MonthChangedEventHandler MonthChanged;
        /// <summary>
        /// Notify the MonthChanged event
        /// </summary>
        /// <param name="args"></param>
        protected virtual void OnMonthChanged(MonthChangedEventArgs args)
        {
            if (MonthChanged != null)
                MonthChanged(this, args);
            this.pb_Agenda.Focus();
        }
        /// <summary>
        /// Event thrown when the user clicked on a day
        /// </summary>
        [Category("MonthCal - Event")]
        [Description("Event thrown when a day was clicked")]
        public event DayClickedEventHandler DayClicked;
        /// <summary>
        /// Notify a click on a day
        /// </summary>
        /// <param name="args"></param>
        protected virtual void OnDayClicked(DayClickedEventArgs args)
        {
            if (DayClicked != null)
                DayClicked(this, args);
            this.pb_Agenda.Focus();
        }
        #endregion
    }
}
